==> rust-git-checks-v1.0.0.zip: rust-git-checks-v1.0.0/src/checks/invalid_utf8.rs <==
// Copyright 2016 Kitware, Inc.
//
// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
// option. This file may not be copied, modified, or distributed
// except according to those terms.

use super::super::*;

use std::char::REPLACEMENT_CHARACTER;

#[derive(Debug, Default, Clone, Copy)]
/// A check which denies commits which modify files containing special characters.
pub struct InvalidUtf8;

impl InvalidUtf8 {
    /// Create a check which rejects file contents which are invalid utf-8.
    ///
    /// Files may be marked as binary by unsetting the `text` attribute.
    pub fn new() -> Self {
        InvalidUtf8 {}
    }
}

impl Check for InvalidUtf8 {
    fn name(&self) -> &str {
        "invalid-utf8"
    }

    fn check(&self, ctx: &CheckGitContext, commit: &Commit) -> Result<CheckResult> {
        let mut result = CheckResult::new();

        for diff in &commit.diffs {
            match diff.status {
                StatusChange::Added |
                StatusChange::Modified(_) => (),
                _ => continue,
            }

            let diff_attr = try!(ctx.check_attr("diff", diff.name.as_ref()));
            if let AttributeState::Unset = diff_attr {
                // Binary files should not be handled here.
                continue;
            }

            // FIXME: does this work for binary files?
            let patch = match commit.file_patch(&diff.name) {
                Ok(s) => s,
                Err(err) => {
                    result.add_alert(format!("failed to get the diff for file `{}` in commit \
                                              {}: {}.",
                                             diff.name,
                                             commit.sha1_short,
                                             err),
                                     true);
                    continue;
                },
            };

            for line in patch.lines().filter(|line| line.starts_with('+')) {
                if line.contains(REPLACEMENT_CHARACTER) {
                    // Escape instances of backticks and backslashes.
                    let safe_line = line[1..]
                        .replace('\\', "\\\\")
                        .replace('`', "\\`");
                    result.add_error(format!("commit {} not allowed; invalid utf-8 sequence \
                                              added in `{}`: `{}`.",
                                             commit.sha1_short,
                                             diff.name,
                                             safe_line));
                }
            }
        }

        Ok(result)
    }
}

#[cfg(test)]
mod tests {
    use super::InvalidUtf8;
    use super::super::test::*;

    static BAD_TOPIC: &'static str = "cf16b71a21023320ffab7b3f7673dc62f33e5022";

    #[test]
    fn test_invalid_utf8() {
        let check = InvalidUtf8;
        let mut conf = GitCheckConfiguration::new();

        conf.add_check(&check);

        let result = test_check("test_invalid_utf8", BAD_TOPIC, &conf);

        assert_eq!(result.warnings().len(), 0);
        assert_eq!(result.alerts().len(), 0);
        assert_eq!(result.errors().len(), 1);
        assert_eq!(result.errors()[0],
                   "commit cf16b71 not allowed; invalid utf-8 sequence added in \
                    `invalid-utf8`: `This file contains an invalid utf-8 sequence: \u{fffd}`.");
        assert_eq!(result.allowed(), false);
        assert_eq!(result.pass(), false);
    }
}

==> rust-git-checks-v2.2.0.zip: rust-git-checks-v2.2.0/src/checks/invalid_utf8.rs <==
// Copyright 2016 Kitware, Inc.
//
// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
// option. This file may not be copied, modified, or distributed
// except according to those terms.

use impl_prelude::*;

use std::char::REPLACEMENT_CHARACTER;

#[derive(Debug, Default, Clone, Copy)]
/// A check which denies commits which modify files containing special characters.
pub struct InvalidUtf8;

impl InvalidUtf8 {
    /// Create a check which rejects file contents which are invalid utf-8.
    ///
    /// Files may be marked as binary by unsetting the `text` attribute.
    pub fn new() -> Self {
        InvalidUtf8 {}
    }
}

impl Check for InvalidUtf8 {
    fn name(&self) -> &str {
        "invalid-utf8"
    }

    fn check(&self, ctx: &CheckGitContext, commit: &Commit) -> Result<CheckResult> {
        let mut result = CheckResult::new();

        for diff in &commit.diffs {
            match diff.status {
                StatusChange::Added |
                StatusChange::Modified(_) => (),
                _ => continue,
            }

            let diff_attr = ctx.check_attr("diff", diff.name.as_path())?;
            if let AttributeState::Unset = diff_attr {
                // Binary files should not be handled here.
                continue;
            }

            // FIXME: does this work for binary files?
            let patch = match commit.file_patch(&diff.name) {
                Ok(s) => s,
                Err(err) => {
                    result.add_alert(format!("failed to get the diff for file `{}` in commit \
                                              {}: {}.",
                                             diff.name,
                                             commit.sha1,
                                             err),
                                     true);
                    continue;
                },
            };

            for line in patch.lines().filter(|line| line.starts_with('+')) {
                if line.contains(REPLACEMENT_CHARACTER) {
                    // Escape instances of backticks and backslashes.
                    let safe_line = line[1..]
                        .replace('\\', "\\\\")
                        .replace('`', "\\`");
                    result.add_error(format!("commit {} not allowed; invalid utf-8 sequence \
                                              added in `{}`: `{}`.",
                                             commit.sha1,
                                             diff.name,
                                             safe_line));
                }
            }
        }

        Ok(result)
    }
}

#[cfg(test)]
mod tests {
    use checks::InvalidUtf8;
    use checks::test::*;

    static BAD_TOPIC: &'static str = "cf16b71a21023320ffab7b3f7673dc62f33e5022";

    #[test]
    fn test_invalid_utf8() {
        let check = InvalidUtf8;
        let mut conf = GitCheckConfiguration::new();

        conf.add_check(&check);

        let result = test_check("test_invalid_utf8", BAD_TOPIC, &conf);

        assert_eq!(result.warnings().len(), 0);
        assert_eq!(result.alerts().len(), 0);
        assert_eq!(result.errors().len(), 1);
        assert_eq!(result.errors()[0],
                   "commit cf16b71a21023320ffab7b3f7673dc62f33e5022 not allowed; invalid utf-8 \
                    sequence added in `invalid-utf8`: `This file contains an invalid utf-8 \
                    sequence: \u{fffd}`.");
        assert_eq!(result.temporary(), false);
        assert_eq!(result.allowed(), false);
        assert_eq!(result.pass(), false);
    }
}
